/*
 * Copyright (C) Mellanox Technologies, Ltd. 2015-2018 ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of Mellanox Technologies, Ltd.
 * (the "Company") and all right, title, and interest in and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef _MLXSW_ITEM_H
#define _MLXSW_ITEM_H

#include <linux/types.h>
#include <linux/string.h>
#include <asm/byteorder.h>
#include <complib/cl_types.h>
#include "bitops.h"

#define cpu_to_be64 __cpu_to_be64
#define cpu_to_be32 __cpu_to_be32
#define cpu_to_be16 __cpu_to_be16

#define be16_to_cpu __be16_to_cpu
#define be32_to_cpu __be32_to_cpu
#define be64_to_cpu __be64_to_cpu

#define BITS_PER_LONG_LONG 64

/*
 * Create a contiguous bitmask starting at bit position @l and ending at
 * position @h. For example
 * GENMASK_ULL(39, 21) gives us the 64bit vector 0x000000ffffe00000.
 */
#define GENMASK(h, l) \
    (((~0UL) << (l)) & (~0UL >> (BITS_PER_LONG - 1 - (h))))

#define GENMASK_ULL(h, l) \
    (((~0ULL) << (l)) & (~0ULL >> (BITS_PER_LONG_LONG - 1 - (h))))

struct mlxsw_item {
    unsigned short offset;           /* bytes in container */
    unsigned short step;           /* step in bytes for indexed items */
    unsigned short in_step_offset;    /* offset within one step */
    unsigned char  shift;          /* shift in bits */
    unsigned char  element_size;      /* size of element in bit array */
    unsigned char  no_real_shift;
    union {
        unsigned char  bits;
        unsigned short bytes;
    } size;
    const char *name;
};
static inline unsigned int __mlxsw_item_offset(const struct mlxsw_item *item, unsigned short index, size_t typesize)
{
    assert(!index || item->step);
    if ((item->offset % typesize != 0) ||
        (item->step % typesize != 0) ||
        (item->in_step_offset % typesize != 0)) {
        assert(FALSE);
    }

    return ((item->offset + item->step * index + item->in_step_offset) /
            typesize);
}

static inline uint8_t __mlxsw_item_get8(const char *buf, const struct mlxsw_item *item, unsigned short index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(uint8_t));
    uint8_t     *b = (uint8_t*)buf;
    uint8_t      tmp;

    tmp = b[offset];
    tmp >>= item->shift;
    tmp &= GENMASK(item->size.bits - 1, 0);
    if (item->no_real_shift) {
        tmp <<= item->shift;
    }
    return tmp;
}

static inline void __mlxsw_item_set8(char *buf, const struct mlxsw_item *item, unsigned short index, uint8_t val)
{
    unsigned int offset = __mlxsw_item_offset(item, index,
                                              sizeof(uint8_t));
    uint8_t *b = (uint8_t*)buf;
    uint8_t  mask = GENMASK(item->size.bits - 1, 0) << item->shift;
    uint8_t  tmp;

    if (!item->no_real_shift) {
        val <<= item->shift;
    }
    val &= mask;
    tmp = b[offset];
    tmp &= ~mask;
    tmp |= val;
    b[offset] = tmp;
}

static inline uint16_t __mlxsw_item_get16(const char *buf, const struct mlxsw_item *item, unsigned short index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(uint16_t));
    __be16      *b = (__be16*)buf;
    uint16_t     tmp;

    tmp = be16_to_cpu(b[offset]);
    tmp >>= item->shift;
    tmp &= GENMASK(item->size.bits - 1, 0);
    if (item->no_real_shift) {
        tmp <<= item->shift;
    }
    return tmp;
}

static inline void __mlxsw_item_set16(char *buf, const struct mlxsw_item *item, unsigned short index, uint16_t val)
{
    unsigned int offset = __mlxsw_item_offset(item, index,
                                              sizeof(uint16_t));
    __be16  *b = (__be16*)buf;
    uint16_t mask = GENMASK(item->size.bits - 1, 0) << item->shift;
    uint16_t tmp;

    if (!item->no_real_shift) {
        val <<= item->shift;
    }
    val &= mask;
    tmp = be16_to_cpu(b[offset]);
    tmp &= ~mask;
    tmp |= val;
    b[offset] = cpu_to_be16(tmp);
}

static inline uint32_t __mlxsw_item_get32(const char *buf, const struct mlxsw_item *item, unsigned short index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(uint32_t));
    __be32      *b = (__be32*)buf;
    uint32_t     tmp;

    tmp = be32_to_cpu(b[offset]);
    tmp >>= item->shift;
    tmp &= GENMASK(item->size.bits - 1, 0);
    if (item->no_real_shift) {
        tmp <<= item->shift;
    }
    return tmp;
}

static inline void __mlxsw_item_set32(char *buf, const struct mlxsw_item *item, unsigned short index, uint32_t val)
{
    unsigned int offset = __mlxsw_item_offset(item, index,
                                              sizeof(uint32_t));
    __be32  *b = (__be32*)buf;
    uint32_t mask = GENMASK(item->size.bits - 1, 0) << item->shift;
    uint32_t tmp;

    if (!item->no_real_shift) {
        val <<= item->shift;
    }
    val &= mask;
    tmp = be32_to_cpu(b[offset]);
    tmp &= ~mask;
    tmp |= val;
    b[offset] = cpu_to_be32(tmp);
}

static inline uint64_t __mlxsw_item_get64(const char *buf, const struct mlxsw_item *item, unsigned short index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(uint64_t));
    __be64      *b = (__be64*)buf;
    uint64_t     tmp;

    tmp = be64_to_cpu(b[offset]);
    tmp >>= item->shift;
    tmp &= GENMASK_ULL(item->size.bits - 1, 0);
    if (item->no_real_shift) {
        tmp <<= item->shift;
    }
    return tmp;
}

static inline void __mlxsw_item_set64(char *buf, const struct mlxsw_item *item, unsigned short index, uint64_t val)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(uint64_t));
    __be64      *b = (__be64*)buf;
    uint64_t     mask = GENMASK_ULL(item->size.bits - 1, 0) << item->shift;
    uint64_t     tmp;

    if (!item->no_real_shift) {
        val <<= item->shift;
    }
    val &= mask;
    tmp = be64_to_cpu(b[offset]);
    tmp &= ~mask;
    tmp |= val;
    b[offset] = cpu_to_be64(tmp);
}

static inline void __mlxsw_item_memcpy_from(const char              *buf,
                                            char                    *dst,
                                            const struct mlxsw_item *item,
                                            unsigned short           index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(char));

    memcpy(dst, &buf[offset], item->size.bytes);
}

static inline void __mlxsw_item_memcpy_to(char                    *buf,
                                          const char              *src,
                                          const struct mlxsw_item *item,
                                          unsigned short           index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(char));

    memcpy(&buf[offset], src, item->size.bytes);
}

static inline char * __mlxsw_item_data(char *buf, const struct mlxsw_item *item, unsigned short index)
{
    unsigned int offset = __mlxsw_item_offset(item, index, sizeof(char));

    return &buf[offset];
}

static inline uint16_t __mlxsw_item_bit_array_offset(const struct mlxsw_item *item, uint16_t index, uint8_t *shift)
{
    uint16_t max_index, be_index;
    uint16_t offset;        /* byte offset inside the array */
    uint8_t  in_byte_index;

    assert(!index || item->element_size);
    if ((item->offset % sizeof(uint32_t) != 0) ||
        (BITS_PER_BYTE % item->element_size != 0)) {
        assert(FALSE);
    }

    max_index = (item->size.bytes << 3) / item->element_size - 1;
    be_index = max_index - index;
    offset = be_index * item->element_size >> 3;
    in_byte_index = index % (BITS_PER_BYTE / item->element_size);
    *shift = in_byte_index * item->element_size;

    return item->offset + offset;
}

static inline uint8_t __mlxsw_item_bit_array_get(const char *buf, const struct mlxsw_item *item, uint16_t index)
{
    uint8_t  shift, tmp;
    uint16_t offset = __mlxsw_item_bit_array_offset(item, index, &shift);

    tmp = buf[offset];
    tmp >>= shift;
    tmp &= GENMASK(item->element_size - 1, 0);
    return tmp;
}

static inline void __mlxsw_item_bit_array_set(char *buf, const struct mlxsw_item *item, uint16_t index, uint8_t val)
{
    uint8_t  shift, tmp;
    uint16_t offset = __mlxsw_item_bit_array_offset(item, index, &shift);
    uint8_t  mask = GENMASK(item->element_size - 1, 0) << shift;

    val <<= shift;
    val &= mask;
    tmp = buf[offset];
    tmp &= ~mask;
    tmp |= val;
    buf[offset] = tmp;
}

#define __ITEM_NAME(_type, _cname, _iname) \
    mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _item

/* _type: cmd_mbox, reg, etc.
 * _cname: containter name (e.g. command name, register name)
 * _iname: item name within the container
 */

#define MLXSW_ITEM8(_type, _cname, _iname, _offset, _shift, _sizebits)                              \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                 \
        .offset = _offset,                                                                          \
        .shift = _shift,                                                                            \
        .size = {.bits = _sizebits, },                                                              \
        .name = #_type "_" #_cname "_" #_iname,                                                     \
    };                                                                                              \
    uint8_t                  mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf) \
    {                                                                                               \
        return __mlxsw_item_get8(buf, &__ITEM_NAME(_type, _cname, _iname), 0);                      \
    }                                                                                               \
    void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, uint8_t val)              \
    {                                                                                               \
        __mlxsw_item_set8(buf, &__ITEM_NAME(_type, _cname, _iname), 0, val);                        \
    }

#define MLXSW_ITEM8_INDEXED(_type, _cname, _iname, _offset, _shift, _sizebits,                           \
                            _step, _instepoffset, _norealshift)                                          \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                      \
        .offset = _offset,                                                                               \
        .step = _step,                                                                                   \
        .in_step_offset = _instepoffset,                                                                 \
        .shift = _shift,                                                                                 \
        .no_real_shift = _norealshift,                                                                   \
        .size = {.bits = _sizebits, },                                                                   \
        .name = #_type "_" #_cname "_" #_iname,                                                          \
    };                                                                                                   \
    uint8_t                  mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf,      \
                                                                                   unsigned short index) \
    {                                                                                                    \
        return __mlxsw_item_get8(buf, &__ITEM_NAME(_type, _cname, _iname),                               \
                                 index);                                                                 \
    }                                                                                                    \
    void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, unsigned short index,          \
                                                               uint8_t val)                              \
    {                                                                                                    \
        __mlxsw_item_set8(buf, &__ITEM_NAME(_type, _cname, _iname),                                      \
                          index, val);                                                                   \
    }

#define MLXSW_ITEM16(_type, _cname, _iname, _offset, _shift, _sizebits)                             \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                 \
        .offset = _offset,                                                                          \
        .shift = _shift,                                                                            \
        .size = {.bits = _sizebits, },                                                              \
        .name = #_type "_" #_cname "_" #_iname,                                                     \
    };                                                                                              \
    uint16_t                 mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf) \
    {                                                                                               \
        return __mlxsw_item_get16(buf, &__ITEM_NAME(_type, _cname, _iname), 0);                     \
    }                                                                                               \
    void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, uint16_t val)             \
    {                                                                                               \
        __mlxsw_item_set16(buf, &__ITEM_NAME(_type, _cname, _iname), 0, val);                       \
    }

#define MLXSW_ITEM16_INDEXED(_type, _cname, _iname, _offset, _shift, _sizebits,                      \
                             _step, _instepoffset, _norealshift)                                     \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                  \
        .offset = _offset,                                                                           \
        .step = _step,                                                                               \
        .in_step_offset = _instepoffset,                                                             \
        .shift = _shift,                                                                             \
        .no_real_shift = _norealshift,                                                               \
        .size = {.bits = _sizebits, },                                                               \
        .name = #_type "_" #_cname "_" #_iname,                                                      \
    };                                                                                               \
    uint16_t                                                                                         \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf, unsigned short index) \
    {                                                                                                \
        return __mlxsw_item_get16(buf, &__ITEM_NAME(_type, _cname, _iname),                          \
                                  index);                                                            \
    }                                                                                                \
    void                                                                                             \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, unsigned short index,       \
                                                              uint16_t val)                          \
    {                                                                                                \
        __mlxsw_item_set16(buf, &__ITEM_NAME(_type, _cname, _iname),                                 \
                           index, val);                                                              \
    }

#define MLXSW_ITEM32(_type, _cname, _iname, _offset, _shift, _sizebits)                             \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                 \
        .offset = _offset,                                                                          \
        .shift = _shift,                                                                            \
        .size = {.bits = _sizebits, },                                                              \
        .name = #_type "_" #_cname "_" #_iname,                                                     \
    };                                                                                              \
    uint32_t                 mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf) \
    {                                                                                               \
        return __mlxsw_item_get32(buf, &__ITEM_NAME(_type, _cname, _iname), 0);                     \
    }                                                                                               \
    void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, uint32_t val)             \
    {                                                                                               \
        __mlxsw_item_set32(buf, &__ITEM_NAME(_type, _cname, _iname), 0, val);                       \
    }

#define MLXSW_ITEM32_INDEXED(_type, _cname, _iname, _offset, _shift, _sizebits,                          \
                             _step, _instepoffset, _norealshift)                                         \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                      \
        .offset = _offset,                                                                               \
        .step = _step,                                                                                   \
        .in_step_offset = _instepoffset,                                                                 \
        .shift = _shift,                                                                                 \
        .no_real_shift = _norealshift,                                                                   \
        .size = {.bits = _sizebits, },                                                                   \
        .name = #_type "_" #_cname "_" #_iname,                                                          \
    };                                                                                                   \
    uint32_t                 mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf,      \
                                                                                   unsigned short index) \
    {                                                                                                    \
        return __mlxsw_item_get32(buf, &__ITEM_NAME(_type, _cname, _iname),                              \
                                  index);                                                                \
    }                                                                                                    \
    void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, unsigned short index,          \
                                                               uint32_t val)                             \
    {                                                                                                    \
        __mlxsw_item_set32(buf, &__ITEM_NAME(_type, _cname, _iname),                                     \
                           index, val);                                                                  \
    }

#define MLXSW_ITEM64(_type, _cname, _iname, _offset, _shift, _sizebits)                               \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                   \
        .offset = _offset,                                                                            \
        .shift = _shift,                                                                              \
        .size = {.bits = _sizebits, },                                                                \
        .name = #_type "_" #_cname "_" #_iname,                                                       \
    };                                                                                                \
    static inline uint64_t   mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf)   \
    {                                                                                                 \
        return __mlxsw_item_get64(buf, &__ITEM_NAME(_type, _cname, _iname), 0);                       \
    }                                                                                                 \
    static inline void mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, uint64_t val) \
    {                                                                                                 \
        __mlxsw_item_set64(buf, &__ITEM_NAME(_type, _cname, _iname), 0,    val);                      \
    }

#define MLXSW_ITEM64_INDEXED(_type, _cname, _iname, _offset, _shift,                                 \
                             _sizebits, _step, _instepoffset, _norealshift)                          \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                  \
        .offset = _offset,                                                                           \
        .step = _step,                                                                               \
        .in_step_offset = _instepoffset,                                                             \
        .shift = _shift,                                                                             \
        .no_real_shift = _norealshift,                                                               \
        .size = {.bits = _sizebits, },                                                               \
        .name = #_type "_" #_cname "_" #_iname,                                                      \
    };                                                                                               \
    uint64_t                                                                                         \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf, unsigned short index) \
    {                                                                                                \
        return __mlxsw_item_get64(buf, &__ITEM_NAME(_type, _cname, _iname),                          \
                                  index);                                                            \
    }                                                                                                \
    void                                                                                             \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, unsigned short index,       \
                                                              uint64_t val)                          \
    {                                                                                                \
        __mlxsw_item_set64(buf, &__ITEM_NAME(_type, _cname, _iname),                                 \
                           index, val);                                                              \
    }

#define MLXSW_ITEM_BUF(_type, _cname, _iname, _offset, _sizebytes)                                \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                               \
        .offset = _offset,                                                                        \
        .size = {.bytes = _sizebytes, },                                                          \
        .name = #_type "_" #_cname "_" #_iname,                                                   \
    };                                                                                            \
    void                                                                                          \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _memcpy_from(const char *buf, char *dst) \
    {                                                                                             \
        __mlxsw_item_memcpy_from(buf, dst,                                                        \
                                 &__ITEM_NAME(_type, _cname, _iname), 0);                         \
    }                                                                                             \
    void                                                                                          \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _memcpy_to(char *buf, const char *src)   \
    {                                                                                             \
        __mlxsw_item_memcpy_to(buf, src,                                                          \
                               &__ITEM_NAME(_type, _cname, _iname), 0);                           \
    }                                                                                             \
    static inline char *                                                                          \
    mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _data(char *buf)                             \
    {                                                                                             \
        return __mlxsw_item_data(buf, &__ITEM_NAME(_type, _cname, _iname), 0);                    \
    }

#define MLXSW_ITEM_BUF_INDEXED(_type, _cname, _iname, _offset, _sizebytes,                  \
                               _step, _instepoffset)                                        \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                         \
        .offset = _offset,                                                                  \
        .step = _step,                                                                      \
        .in_step_offset = _instepoffset,                                                    \
        .size = {.bytes = _sizebytes, },                                                    \
        .name = #_type "_" #_cname "_" #_iname,                                             \
    };                                                                                      \
    void                                                                                    \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _memcpy_from(const char *buf,      \
                                                                      unsigned short index, \
                                                                      char *dst)            \
    {                                                                                       \
        __mlxsw_item_memcpy_from(buf, dst,                                                  \
                                 &__ITEM_NAME(_type, _cname, _iname), index);               \
    }                                                                                       \
    void                                                                                    \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _memcpy_to(char *buf,              \
                                                                    unsigned short index,   \
                                                                    const char *src)        \
    {                                                                                       \
        __mlxsw_item_memcpy_to(buf, src,                                                    \
                               &__ITEM_NAME(_type, _cname, _iname), index);                 \
    }                                                                                       \
    static inline char *                                                                    \
    mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _data(char *buf, unsigned short index) \
    {                                                                                       \
        return __mlxsw_item_data(buf,                                                       \
                                 &__ITEM_NAME(_type, _cname, _iname), index);               \
    }

#define MLXSW_ITEM_BIT_ARRAY(_type, _cname, _iname, _offset, _sizebytes,                              \
                             _element_size)                                                           \
    static struct mlxsw_item __ITEM_NAME(_type, _cname, _iname) = {                                   \
        .offset = _offset,                                                                            \
        .element_size = _element_size,                                                                \
        .size = {.bytes = _sizebytes, },                                                              \
        .name = #_type "_" #_cname "_" #_iname,                                                       \
    };                                                                                                \
    uint8_t                                                                                           \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _get(const char *buf, uint16_t index)        \
    {                                                                                                 \
        return __mlxsw_item_bit_array_get(buf,                                                        \
                                          &__ITEM_NAME(_type, _cname, _iname),                        \
                                          index);                                                     \
    }                                                                                                 \
    void                                                                                              \
        mlxsw_ ## _type ## _ ## _cname ## _ ## _iname ## _set(char *buf, uint16_t index, uint8_t val) \
    {                                                                                                 \
        return __mlxsw_item_bit_array_set(buf,                                                        \
                                          &__ITEM_NAME(_type, _cname, _iname),                        \
                                          index, val);                                                \
    }                                                                                                 \

#endif /* ifndef _MLXSW_ITEM_H */
